@@ -1,7 +1,6 @@
 package Signals::XSIG;
 
-## no critic (Variables::RequireLocalizedPunctuationVars)
-## no critic (Modules::ProhibitAutomaticExportation)
+## no critic (RequireLocalizedPunctuationVars, ProhibitAutomaticExport)
 
 use Signals::XSIG::Default;
 use Carp;
@@ -9,6 +8,7 @@ use Config;
 use Exporter;
 use POSIX ();
 use Symbol qw(qualify);
+use Time::HiRes 'time';
 use warnings;
 use strict;
 
@@ -19,18 +19,26 @@ our @EXPORT_OK = qw(untied %DEFAULT_BEHAVIOR);
 our %DEFAULT_BEHAVIOR;
 *DEFAULT_BEHAVIOR = \%Signals::XSIG::Default::DEFAULT_BEHAVIOR;
 
-our $VERSION = '0.15_02';
+our $VERSION = '0.15_03';
 our (%XSIG, %_XSIG, %SIGTABLE, $_REFRESH, $_DISABLE_WARNINGS);
 our $_INITIALIZED = 0;
 our $SIGTIE = bless {}, 'Signals::XSIG::TieSIG';
 our $XSIGTIE = bless {}, 'Signals::XSIG::TieXSIG';
 our $TIEARRAY_CLASS = 'Signals::XSIG::TieArray';
-our $XDEBUG = 0;  # to be removed when we get a handle on t/02 failures
 
 my %TIEDSCALARS = (); # map signal names to S::X::TiedScalar objs
 my %alias = ();
 
-BEGIN {                 ## no critic(StringyEval)
+
+###
+# to be removed when we get a handle on intermittent t/02 failures
+our ($XDEBUG,@XLOG) = (0);
+sub XLOG{no warnings"uninitialized";push @XLOG,(Time::HiRes::time-$^T)." @_"}
+sub XLOG_DUMP{join"\n",@XLOG}
+###
+
+
+BEGIN {
     # how to detect global destruction. Idea from Devel::GlobalDestruction 0.13
     if (defined ${^GLOBAL_PHASE}) {
         eval 'sub __inGD() {${^GLOBAL_PHASE} eq "DESTRUCT" && __END()}; 1';
@@ -64,9 +72,10 @@ sub _init {
 
     foreach my $sig (@name, '__WARN__', '__DIE__') {
 	tie $_XSIG{$sig}, 'Signals::XSIG::TieScalar', $sig;
-	$_XSIG{$sig} = [];
-	tie @{$_XSIG{$sig}}, $TIEARRAY_CLASS, $sig;
-	$_XSIG{$sig}[0] = $SIG{$sig};
+#### this functionality all happens within S::X::TieScalar::TIESCALAR now
+#	$_XSIG{$sig} = [];
+#	tie @{$_XSIG{$sig}}, $TIEARRAY_CLASS, $sig;
+#	$_XSIG{$sig}[0] = $SIG{$sig};
     }
     tie %SIG, 'Signals::XSIG::TieSIG';
     $_REFRESH = 1;
@@ -101,9 +110,8 @@ sub __shadow__die__handler  {          ## no critic (Unpacking)
 END { &__END; }
 
 sub __END {
-    no warnings 'redefine';  ## no critic (NoWarnings)
+    no warnings 'redefine';
     *__inGD = sub () { 1 };
-    return;
 }
 
 sub __shadow_signal_handler {
@@ -232,10 +240,8 @@ sub _qualify_handler {
 # Associates $SIG{key} with $XSIG{$sig}[0]
 #
 
-use Time::HiRes 'time';
-
 sub Signals::XSIG::TieSIG::TIEHASH {
-    $XDEBUG && push @{$SIGTIE->{DEBUG_LOG}},[TIE=>[],time-$^T,[caller]];
+    $XDEBUG && XLOG("TIE caller=",caller);
     return $SIGTIE;
 }
 
@@ -244,11 +250,13 @@ sub Signals::XSIG::TieSIG::FETCH {
 
     # track function calls to diagnose test failures
     $XDEBUG && $key ne '__DIE__'
-            && push @{$self->{DEBUG_LOG}},[FETCH => [$key],time-$^T,[caller]];
+            && XLOG("FETCH key=$key caller=",caller);
     if (_resolve_signal($key)) {
+        $XDEBUG && $key ne '__DIE__' && XLOG("    result(1)=",$_XSIG{$key}[0]);
 	return $_XSIG{$key}[0];
     } else {
 	my $r = untied { $SIG{$key} };
+        $XDEBUG && $key ne '__DIE__' && XLOG("    result(2)=$r");
 	return $r;
     }
 }
@@ -256,13 +264,14 @@ sub Signals::XSIG::TieSIG::FETCH {
 sub Signals::XSIG::TieSIG::STORE {
     my ($self,$key,$value) = @_;
     $XDEBUG && $key ne '__DIE__'
-            && push @{$self->{DEBUG_LOG}},[STORE => [$key,$value],time-$^T,[caller]];
+        && XLOG("STORE($key,$value) caller=",caller);
     if (_resolve_signal($key)) {
 	my $old = $_XSIG{$key}[0];
 	$_XSIG{$key}[0] = $value;
 	return $old;
     } else {
 	my $old;
+        $XDEBUG && XLOG("    key $key not resolved");
 	untied {
 	    no warnings 'signal';          ## no critic (NoWarnings)
 	    $old = $SIG{$key};
@@ -275,13 +284,15 @@ sub Signals::XSIG::TieSIG::STORE {
 sub Signals::XSIG::TieSIG::DELETE {
     my ($self,$key) = @_;
     $XDEBUG && $key ne '__DIE__'
-            && push @{$self->{DEBUG_LOG}},[DELETE=>[$key],time-$^T,[caller]];
+        && XLOG("DELETE key=$key caller=",caller);
     if (_resolve_signal($key)) {
 	my $old = $_XSIG{$key}[0];
+        $XDEBUG && XLOG("    DELETE result(1)=$old");
 	$_XSIG{$key}[0] = undef;
 	return $old;
     } else {
 	my $old = $self->FETCH($key);
+        $XDEBUG && XLOG("    DELETE result(2)=$old");
 	untied { $SIG{$key} = undef; delete $SIG{$key} };
 	return $old;
     }
@@ -289,34 +300,50 @@ sub Signals::XSIG::TieSIG::DELETE {
 
 sub Signals::XSIG::TieSIG::CLEAR {
     my ($self) = @_;
-    $XDEBUG && push @{$self->{DEBUG_LOG}},[CLEAR=>[],time-$^T,[caller]];
+    $XDEBUG && do {
+        XLOG("CLEAR caller=",caller);
+        XLOG("    CLEAR init USR1=",@{$_XSIG{USR1}});
+        XLOG("    CLEAR init USR2=",@{$_XSIG{USR2}});
+    };
+
+    # $_XSIG{$_} is tied to 'Signals::XSIG::TieScalar'
+    # @{$_XSIG{$_}} is tied to 'Signals::XSIG::TieArray'
+    #     a tied array gives us:
+    #     1. negative indices, used appropriately
+    #     2. immediate update to the real signal handler
+    # $_XSIG{$_}[0] is equiv to
+    #     Signals::XSIG::TieArray::STORE( tied(@{$_XSIG{$_}}, 0, undef) )
     $_XSIG{$_}[0] = undef for keys %_XSIG;
+    $XDEBUG && do {
+        XLOG("    CLEAR final USR1=",@{$_XSIG{USR1}});
+        XLOG("    CLEAR final USR2=",@{$_XSIG{USR2}});
+    };
     return;
 }
 
 sub Signals::XSIG::TieSIG::EXISTS {
     my ($self,$key) = @_;
     $XDEBUG && $key ne '__DIE__'
-            && push @{$self->{DEBUG_LOG}},[EXISTS=>[$key],time-$^T,[caller]];
+        && XLOG("EXISTS key=$key caller=",caller);
     return untied { exists $SIG{$key} };
 }
 
 sub Signals::XSIG::TieSIG::FIRSTKEY {
     my ($self) = @_;
-    $XDEBUG && push @{$self->{DEBUG_LOG}},[FIRSTKEY=>[],time-$^T,[caller]];
+    $XDEBUG && XLOG("FIRSTKEY caller=",caller);
     my $a = keys %_XSIG;
+    $XDEBUG && XLOG("    FIRSTKEY result=$a");
     return each %_XSIG;
 }
 
 sub Signals::XSIG::TieSIG::NEXTKEY {
     my ($self, $lastkey) = @_;
-    $XDEBUG && push @{$self->{DEBUG_LOG}},[NEXTKEY=>[],time-$^T,[caller]];
+    $XDEBUG && XLOG("NEXTKEY lastkey=$lastkey caller=",caller);
     return each %_XSIG;
 }
 
-sub Signals::XSIG::TieSIG::UNTIE { ## no critic(Unpacking)
-    $XDEBUG && !__inGD()
-            && push @{$_[0]->{DEBUG_LOG}},[UNTIE=>[$_[1]],time-$^T,[caller]];
+sub Signals::XSIG::TieSIG::UNTIE {
+    $XDEBUG && !__inGD() && XLOG("UNTIE caller=",caller);
     return;
 }
 
@@ -361,6 +388,7 @@ sub Signals::XSIG::TieArray::STORE {
 
     $handler = _qualify_handler($handler);
     $self->{handlers}[$index] = $handler;
+    $XDEBUG && XLOG("TA::STORE key=$self->{key} index=$index val=",$handler," caller=",caller);
     $self->_refresh_SIG();
     return $old;
 }
@@ -405,6 +433,7 @@ sub Signals::XSIG::TieArray::_refresh_SIG {
     }
     untied {
 	no warnings qw(uninitialized signal); ## no critic (NoWarnings)
+        $XDEBUG && XLOG("_refresh_SIG key=$self->{key} handler=$handler_to_install");
 	$SIG{$sig} = $handler_to_install;
     };
     return;
@@ -496,8 +525,8 @@ sub Signals::XSIG::TieArray::SPLICE { }
 # For tie-ing $XSIG{signal}.
 # Handles expressions like  $XSIG{signal} = [ ... ]
 # and                       $XSIG{signal} = handler
-# Main purpose is to assert that $XSIG{$sig} is a reference
-# to a Signals::XSIG::TieArray object.
+# Main purpose is to assert that $XSIG{$sig} is always set
+# to a reference to a  Signals::XSIG::TieArray  object.
 #
 
 sub Signals::XSIG::TieScalar::TIESCALAR {
@@ -510,6 +539,7 @@ sub Signals::XSIG::TieScalar::TIESCALAR {
     my $self = bless { key => $key }, 'Signals::XSIG::TieScalar';
     $self->{val} = [];
     tie @{$self->{val}}, $TIEARRAY_CLASS, $key;
+    $self->{val}[0] = $SIG{$key};
     $TIEDSCALARS{$key} = $self;
     return $self;
 }
@@ -519,7 +549,6 @@ sub Signals::XSIG::TieScalar::FETCH {
     my $key = my $key2 = $self->{key};
     Signals::XSIG::_resolve_signal($key);
     if ($key ne $key2 && !$self->{copied}) {
-        #print STDERR "\$XSIG{$key2\} copied from \$XSIG{$key}\n";
 	$self->{val} = (tied $Signals::XSIG::_XSIG{$key})->FETCH;
 	$self->{copied} = $key;
         push @{(tied $Signals::XSIG::_XSIG{$key})->{aliases}}, $key2;
@@ -529,7 +558,7 @@ sub Signals::XSIG::TieScalar::FETCH {
     return $self->{val};
 }
 
-# $XSIG{key} = [ LIST ]   ==>  store LIST, bless LIST as TieArray
+# $XSIG{key} = [ LIST ]   ==>  store LIST, tie LIST as TieArray
 # $XSIG{key} = EXPR       ==>  treat as  $SIG{key}=EXPR,$XSIG{key}[0]=EXPR
 sub Signals::XSIG::TieScalar::STORE {
     my ($self, $value) = @_;
@@ -539,21 +568,11 @@ sub Signals::XSIG::TieScalar::STORE {
 	$value = [ $value ];
     }
 
-    if (ref $value eq 'ARRAY') {
-	my $key = $self->{key};
-	$self->{val} = [];
-	tie @{$self->{val}}, 
-	    $TIEARRAY_CLASS,
-	    $self->{key}, @$value;
-	(tied @{$self->{val}})->_refresh_SIG;
-	return $old;
-    }
-
-    croak "Thought this code was unreachable.\n";
-
-    # otherwise, treat  $XSIG{key}=VAL  like $SIG{key}=VAL or $XSIG{key}[0]=val
-    #$self->{val}[0] = $value;
-    #return $old;
+    my $key = $self->{key};
+    $self->{val} = [];
+    tie @{$self->{val}}, $TIEARRAY_CLASS, $self->{key}, @$value;
+    (tied @{$self->{val}})->_refresh_SIG;
+    return $old;
 }
 
 ##################################################################
@@ -561,8 +580,9 @@ sub Signals::XSIG::TieScalar::STORE {
 # Signals::XSIG::TieXSIG
 #
 # Only for tie-ing %XSIG.
-# adds behavior to %XSIG hash so we can make assignments to it
-# so that @{$XSIG{$sig}} is a S::X::TieArray
+# adds behavior to %XSIG hash so we can make assignments to
+# $XSIG{sig} so that @{$XSIG{$sig}} is always a
+# Signals::XSIG::TieArray
 #
 
 sub Signals::XSIG::TieXSIG::TIEHASH {
@@ -642,7 +662,7 @@ Signals::XSIG - install multiple signal handlers through %XSIG
 
 =head1 VERSION
 
-Version 0.15_02
+Version 0.15_03
 
 =head1 SYNOPSIS
 
